package PrinterDialogForm;
use Qt;
use PrinterDialogFormGUI;
use Qt::isa "PrinterDialogFormGUI";
use FileHandle;
use Cwd;
use PrintOptionsDialogForm;
use Radados::showEvent;
use Qt::slots
	OnPreview => [],
	OnPrint => [],
	OnShowOptions => [],
	OnCleanClose => [],
	closeEvent => [ 'QEvent*' ],
	OnSelectOutputFile => [],
	OnChangeOptions => [];

sub NEW
{
	my $self = shift;
	$self->SUPER::NEW(@_);
	this->{FileToPrint} = "";
	this->{SetPrintJobTitle} = "";
	this->{AppPid} = 0;
	this->{TempFiles} = ();
	this->{Orientation} = "portrait";
	this->{PageSize} = "letter";
	GetListOfPrinters();
	PrintButton->setFocus();
	this->{OptionsForm} = PrintOptionsDialogForm;
	this->connect(PreviewButton, SIGNAL('clicked()'), SLOT('OnPreview()'));
	this->connect(PrintButton, SIGNAL('clicked()'), SLOT('OnPrint()'));
	this->connect(CancelButton, SIGNAL('clicked()'), SLOT('OnCleanClose()'));
	this->connect(OptionsButton, SIGNAL('clicked()'), SLOT('OnShowOptions()'));
	this->connect(this->{OptionsForm}->OkButton, SIGNAL('clicked()'), SLOT('OnChangeOptions()'));
	this->connect(SelectOutputButton, SIGNAL('clicked()'), SLOT('OnSelectOutputFile()'));	
	OutPutEdit->setText($ENV{HOME}."/print.pdf");
}

sub OnPrint
{
	my $PostScriptFile = ConvertFileToPostScript();
	
	my $PrinterToUse = comboBox1->currentText();
	if($PrinterToUse ne "PDF generator")
	{
		my $PrintCommand = "lp -d ".comboBox1->currentText()." $PostScriptFile";
		if(this->{SetPrintJobTitle} ne "")
		{
			$PrintCommand .= " -t \"".this->{SetPrintJobTitle}."\"";
		}
		RunCommand($PrintCommand);
	} else
	{		
		$PdfOutPut = OutPutEdit->text();
		print "ps2pdf $PostScriptFile $PdfOutPut\n";
		RunCommand("ps2pdf $PostScriptFile $PdfOutPut");
	}
	if(KeepOpenCheckBox->isChecked() == 0)
	{
		print "Call close\n";
		OnCleanClose();
	}
}

sub OnSelectOutputFile
{
	$SaveAsFileName = Qt::FileDialog::getSaveFileName(OutPutEdit->text(), "Portable Document Format (*.pdf)", this, "Print to PDF", "Enter a file name");
	if($SaveAsFileName ne "")
	{
		if(index($SaveAsFileName, ".pdf", 0) != length($SaveAsFileName) - 4)
		{
			$SaveAsFileName .= ".pdf";
		}
		OutPutEdit->setText($SaveAsFileName);
	}
}

sub OnChangeOptions
{
	if(this->{OptionsForm}->{PortraitButton}->isChecked() == 1)
	{
		this->{Orientation} = "portrait";
	} else
	{
		this->{Orientation} = "landscape";
	}
	if(this->{OptionsForm}->{comboBox1}->currentText() eq "US Letter")
	{
		this->{PageSize} = "letter";
	} else
	{
		this->{PageSize} = "legal";		
	}
	this->{OptionsForm}->close();
}

sub OnShowOptions
{
	this->{OptionsForm}->SetOrientation(this->{Orientation});
	this->{OptionsForm}->SetPageSize(this->{PageSize});
	this->{OptionsForm}->show;
}

sub GetPrintableFileType
{
	my $Command = "file ".this->{FileToPrint};
	my $FileType = `$Command`;
	$FileType =~ s/\n//;
	print "raw FileType = $FileType\n";
	if(index($FileType, "HTML", 0) > -1)
	{
		return("HTML");
	}
	if(index($FileType, "PostScript", 0) > -1)
	{
		return("PS");
	}
	if(index($FileType, "ASCII", 0) > -1)
	{
		return("TEXT");
	}
	if(index($FileType, "text", 0) > -1)
	{
		return("TEXT");
	}	
}

sub ConvertFileToPostScript
{
	my $FileType = GetPrintableFileType();
	print "FileType = $FileType\n";
	my $SourceFile;
	my $FormatFile = "/etc/printdlg/".this->{PageSize}.".".this->{Orientation};
	my $TempFile = "";
	
	print "Look for config = $FormatFile\n";
	if(! -f $FormatFile)
	{
		Qt::MessageBox::critical(this, "Error", "Missing configuration file.\n/etc/printdlg/".this->{PageSize}.".".this->{Orientation}, &Qt::MessageBox::Ok, 0);
		OnCleanClose();						
	}
	
	$SourceFile = this->{FileToPrint};
	if($FileType eq "TEXT")
	{	
		$TempFile = GetATempFileName();
		$TempTextToHtml = "<html><head><title>".this->{SetPrintJobTitle}."</title></head><body><pre>";
		$TempTextToHtml .= `cat $SourceFile`;
		$TempTextToHtml .= "</pre></body></html>";
		print "Convert TEXT to HTML $TempFile\n";
		$FilePointer = new FileHandle $TempFile, "w";
      if($FilePointer)
      {
      	print $FilePointer $TempTextToHtml;
         $FilePointer->close;
     	} else
		{
			Qt::MessageBox::critical(this, "Error", "Could not write to temp file.\n$TempFile", &Qt::MessageBox::Ok, 0);
			OnCleanClose();
		}
		$FileType = "HTML";
		$SourceFile = $TempFile;
	}	
	if($FileType eq "HTML")
	{	
		$TempFile = GetATempFileName();
		RunCommand("html2ps -U -f $FormatFile -D -c $SourceFile > $TempFile");		
		this->{FileToPrint} = $TempFile;
		$FileType = "PS";
	}	
	return(this->{FileToPrint});
}

sub GetATempFileName
{
	my $TempReturn = "";
	my $TempDirectory = "";
	
	if(-d this->{AppRootPath}."/tmp")
	{
		$TempDirectory = this->{AppRootPath}."/tmp";
	}	
	if(($TempDirectory eq "") && (-d this->{AppRootPath}."/temp"))
	{
		$TempDirectory = this->{AppRootPath}."/temp";
	}	
	if(($TempDirectory eq "") && (-d "/tmp"))
	{
		$TempDirectory = "/tmp";
	}	
	if($TempDirectory ne "")
	{
		my $TempCounter = 0;
		$TempReturn = $TempDirectory."/printing.tmp.".this->{AppPid}.$TempCounter;
		while(-f $TempReturn)
		{
			$TempCounter++;
			$TempReturn = $TempDirectory."/printing.tmp.".this->{AppPid}.$TempCounter;
		}
		`touch $TempReturn`;
		push @{this->{TempFiles}}, $TempReturn;
	}	
	
	if($TempReturn eq "")
	{
		Qt::MessageBox::critical(this, "Error", "Could not create temp file.", &Qt::MessageBox::Ok, 0);
		OnCleanClose();
	}
	return($TempReturn);
}

sub OnPreview
{
	my $PreviewProgream = GetProgramPath("ggv");
	if($PreviewProgream eq "")
	{
		$PreviewProgream = GetProgramPath("gv");
	}
	if($PreviewProgream eq "")
	{
		Qt::MessageBox::critical(this, "Error", "Cannot open a preview.\nPlease install Gnome Ghost View or Ghost View.", &Qt::MessageBox::Ok, 0);
	}
	my $PrevewFile = ConvertFileToPostScript();
	this->hide();
	RunCommand("$PreviewProgream $PrevewFile");
	if(KeepOpenCheckBox->isChecked() == 0)
	{
		OnCleanClose();
	}
	this->show();
}

sub GetProgramPath
{
	my $ProgramToLocate = shift;
	my $ReturnText = "";
	my $ParseText = `whereis $ProgramToLocate | cut -d: -f2- | gawk '{print \$1}'`;
	print "ParseText = $ParseText\n";
	$ParseText =~ s/\n//;
	if(($ParseText ne "") && (-f $ParseText))
	{
		$ReturnText = $ParseText;
	}
	return($ReturnText);
}

sub SetPrintJobTitle
{
	this->{SetPrintJobTitle} = shift;
}

sub SetPrintOptions
{
	my $PrintOptions = shift;
	
	print "SetPrintOptions\n";
	my @Options = split(/,/, $PrintOptions);
	my @BreakDown;
	foreach $KeyOp (@Options)
	{
		@BreakDown = split(/=/, $KeyOp);
		if($BreakDown[0] eq "orientation")
		{
			if($BreakDown[1] eq "portrait")
			{
				this->{Orientation} = "portrait";
			}
			if($BreakDown[1] eq "landscape")
			{
				this->{Orientation} = "landscape";
			}			
		}
		if($BreakDown[0] eq "paper")
		{
			if($BreakDown[1] eq "letter")
			{
				this->{PageSize} = "letter";
			}
			if($BreakDown[1] eq "legal")
			{
				this->{PageSize} = "legal";
			}			
		}
	}
}

sub GetListOfPrinters
{
	comboBox1->clear();
	my $PrintCapFile = "";

	# This maybe a better way to get a list of printer names.
	# lpc status | grep "[A-z0-9]:" | sed -e 's/://g'
	
	# cat of printcap is not a good idea
	# Use lpstat -a for better results
	if(-f "/etc/printcap.cups" )
	{
		$PrintCapFile = "/etc/printcap.cups";
	}
	
	if($PrintCapFile ne "")
	{
	    my $FilePointer = new FileHandle $PrintCapFile, "r";
	    if($FilePointer)
	    {
		my $ParseLine;
		my @PrinterConf;
		while(! $FilePointer->eof)
		{
			$ParseLine = $FilePointer->getline;
			$ParseLine =~ s/\n//;
			if(index($ParseLine, "#", 0) == -1)
			{
				@PrinterConf = split(/\|/, $ParseLine);
				comboBox1->insertItem($PrinterConf[0]);
			}
		}
		$FilePointer->close;
	    }
	}
	comboBox1->insertItem("PDF generator");
}

sub SetFileToPrint
{
	this->{FileToPrint} = shift;
	if(length(this->{FileToPrint}) == 0)
	{
		Qt::MessageBox::critical(this, "Error", "Please give a file name as argument.", &Qt::MessageBox::Ok, 0);
		OnCleanClose();
	}
	
	if(! -f this->{FileToPrint})
	{
		Qt::MessageBox::critical(this, "Error", "Cannot find that file.", &Qt::MessageBox::Ok, 0);
		OnCleanClose();
	}
}

sub SetAppRootPath
{
	my $LocalRunPath = shift;
	my $OrigDir = getcwd();
	chdir($LocalRunPath);
	chdir("../");
	this->{AppRootPath} = getcwd();
	chdir($OrigDir);
}

sub closeEvent
{
	my $EventPointer = shift;
	OnCleanClose();
	$EventPointer->accept();
}

sub OnCleanClose
{
	print "\n----------------------\nsub closeEvent\n";
	
	foreach $TempFile (@{this->{TempFiles}})
	{
		if(-f $TempFile)
		{
			unlink($TempFile);
		}
	}
	this->close();
}

sub RunCommand
{
	my $CommandToRun = shift;
	print "$CommandToRun\n";
	
	my $retval = 0;
	$retval = system($CommandToRun);
	if($retval != 0)
	{
		Qt::MessageBox::critical(this, "Error", "Could not run command\n$CommandToRun", &Qt::MessageBox::Ok, 0);
		if((-d this->{TempPath}) && (this->{DeleteTemp} == 1))
		{
			system("rm -rf ".this->{TempPath});
		}
		OnCleanClose();
	}
}

sub SetApplicationPid
{
	this->{AppPid} = shift;
}

sub OnSelectFile
{
	this->{OutPutFile} = OutPutEdit->text();
	this->{OutPutFile} = Qt::FileDialog::getSaveFileName(this->{OutPutFile}, "*.pdf", this, "Print to PDF", "Enter a file name");
	OutPutEdit->setText(this->{OutPutFile});
}

1;
